#include "stairs.h"
#include "darwin_kinematics.h"
#include "darwin_math.h"
#include "ram.h"
#include <math.h>

typedef enum {SHIFT_WEIGHT_LEFT=0x10,RISE_RIGHT_FOOT=0x20,ADVANCE_RIGHT_FOOT=0x30,CONTACT_RIGHT_FOOT=0x40,SHIFT_WEIGHT_RIGHT=0x50,RISE_LEFT_FOOT=0x60,ADVANCE_LEFT_FOOT=0x70,CONTACT_LEFT_FOOT=0x80,CENTER_WEIGHT=0x90} stairs_phase_t;

// internal motion variables
float stairs_Z_stair_height;
float stairs_Z_overshoot;
float stairs_Y_shift_amplitude;
float stairs_X_shift_amplitude;
float stairs_R_shift_amplitude;
float stairs_P_shift_amplitude;
float stairs_A_shift_amplitude;
float stairs_Y_spread_amplitude;
float stairs_X_shift_body;

// internal offset variables
float stairs_Hip_Pitch_Offset;
float stairs_X_Offset;
float stairs_Y_Offset;
float stairs_Z_Offset;
float stairs_R_Offset;
float stairs_P_Offset;
float stairs_A_Offset;

// internal time variables
float stairs_Time;
float stairs_period;
float stairs_shift_weight_left_time;
float stairs_rise_right_foot_time;
float stairs_advance_right_foot_time;
float stairs_contact_right_foot_time;
float stairs_shift_weight_right_time;
float stairs_rise_left_foot_time;
float stairs_advance_left_foot_time;
float stairs_contact_left_foot_time;
float stairs_center_weight_time;

// control variables
uint8_t stairs_Ctrl_Running;
uint8_t stairs_up;

// private functions

// public functions
void stairs_init(uint16_t period_us)
{
  // initialize the internal motion variables


  // initialize internal control variables
  stairs_Time=0;
  stairs_period=((float)period_us)/1000.0;
  stairs_Ctrl_Running=0x00;
  stairs_up=0x00;
}

void stairs_set_period(uint16_t period_us)
{
  stairs_period=((float)period_us)/1000.0;
}

uint16_t stairs_get_period(void)
{
  return (uint16_t)(stairs_period*1000);
}

void stairs_start(uint8_t up)
{
  // load all parameters
  stairs_shift_weight_left_time=((float)ram_data[DARWIN_STAIRS_PHASE1_TIME_L]+(ram_data[DARWIN_STAIRS_PHASE1_TIME_H]<<8));
  stairs_rise_right_foot_time=((float)ram_data[DARWIN_STAIRS_PHASE2_TIME_L]+(ram_data[DARWIN_STAIRS_PHASE2_TIME_H]<<8));
  stairs_advance_right_foot_time=((float)ram_data[DARWIN_STAIRS_PHASE3_TIME_L]+(ram_data[DARWIN_STAIRS_PHASE3_TIME_H]<<8));
  stairs_contact_right_foot_time=((float)ram_data[DARWIN_STAIRS_PHASE4_TIME_L]+(ram_data[DARWIN_STAIRS_PHASE4_TIME_H]<<8));
  stairs_shift_weight_right_time=((float)ram_data[DARWIN_STAIRS_PHASE5_TIME_L]+(ram_data[DARWIN_STAIRS_PHASE5_TIME_H]<<8));
  stairs_rise_left_foot_time=((float)ram_data[DARWIN_STAIRS_PHASE6_TIME_L]+(ram_data[DARWIN_STAIRS_PHASE6_TIME_H]<<8));
  stairs_advance_left_foot_time=((float)ram_data[DARWIN_STAIRS_PHASE7_TIME_L]+(ram_data[DARWIN_STAIRS_PHASE7_TIME_H]<<8));
  stairs_contact_left_foot_time=((float)ram_data[DARWIN_STAIRS_PHASE8_TIME_L]+(ram_data[DARWIN_STAIRS_PHASE8_TIME_H]<<8));
  stairs_center_weight_time=((float)ram_data[DARWIN_STAIRS_PHASE9_TIME_L]+(ram_data[DARWIN_STAIRS_PHASE9_TIME_H]<<8));
  stairs_Z_stair_height=((float)((int8_t)ram_data[DARWIN_STAIRS_Z_HEIGHT]));
  stairs_Z_overshoot=((float)((int8_t)ram_data[DARWIN_STAIRS_Z_OVERSHOOT]));
  stairs_Y_shift_amplitude=((float)((int8_t)ram_data[DARWIN_STAIRS_Y_SHIFT]));
  stairs_X_shift_amplitude=((float)((int8_t)ram_data[DARWIN_STAIRS_X_SHIFT]));
  stairs_Hip_Pitch_Offset=((float)((int16_t)(ram_data[DARWIN_WALK_HIP_PITCH_OFF_L]+(ram_data[DARWIN_WALK_HIP_PITCH_OFF_H]<<8))))/1024.0;
  stairs_R_shift_amplitude=((float)((int8_t)ram_data[DARWIN_STAIRS_R_SHIFT]))*PI/720.0;// (r_shift/4)*(pi/180)
  stairs_P_shift_amplitude=((float)((int8_t)ram_data[DARWIN_STAIRS_P_SHIFT]))*PI/720.0;// (p_shift/4)*(pi/180)
  stairs_A_shift_amplitude=((float)((int8_t)ram_data[DARWIN_STAIRS_A_SHIFT]))*PI/720.0;// (a_shift/4)*(pi/180)
  stairs_Y_spread_amplitude=((float)((int8_t)ram_data[DARWIN_STAIRS_Y_SPREAD]));
  stairs_X_shift_body=((float)((int8_t)ram_data[DARWIN_STAIRS_X_SHIFT_BODY]));
  stairs_X_Offset=((float)((int8_t)ram_data[DARWIN_STAIRS_X_OFFSET]));
  stairs_Y_Offset=((float)((int8_t)ram_data[DARWIN_STAIRS_Y_OFFSET]));
  stairs_Z_Offset=((float)((int8_t)ram_data[DARWIN_STAIRS_Z_OFFSET]));
  stairs_R_Offset=((float)((int8_t)ram_data[DARWIN_STAIRS_X_OFFSET]))*PI/1440.0;
  stairs_P_Offset=((float)((int8_t)ram_data[DARWIN_STAIRS_Y_OFFSET]))*PI/1440.0;
  stairs_A_Offset=((float)((int8_t)ram_data[DARWIN_STAIRS_Z_OFFSET]))*PI/1440.0;
  // start operation
  ram_data[DARWIN_STAIRS_CNTRL]|=STAIRS_STATUS;
  stairs_Ctrl_Running=0x01;
  stairs_up=up;
}

void stairs_stop(void)
{
  stairs_Ctrl_Running=0x00;
}

uint8_t is_climbing_stairs(void)
{
  if(ram_data[DARWIN_STAIRS_CNTRL]&STAIRS_STATUS)
    return 0x01;
  else 
    return 0x00;
}

uint8_t stairs_get_phase(void)
{
  return (int8_t)ram_data[DARWIN_STAIRS_CNTRL]&STAIRS_PHASE;
}

// motion manager interface functions
void stairs_process(void)
{
  float angle[14]={0},ep[12]={0};
  float delta;

  if(ram_data[DARWIN_STAIRS_CNTRL]&STAIRS_STATUS)
  {
    ram_data[DARWIN_STAIRS_CNTRL]&=(~STAIRS_PHASE);
    if(stairs_up)
    {
      if(stairs_Time>=0 && stairs_Time<=stairs_shift_weight_left_time)
      {
        //1
        delta=stairs_Time/stairs_shift_weight_left_time;
        ep[0]=stairs_X_Offset;
        ep[1]=-stairs_Y_Offset/2.0+stairs_Y_shift_amplitude*delta;
        ep[2]=stairs_Z_Offset;
        ep[3]=stairs_R_Offset;
        ep[4]=stairs_P_Offset;
        ep[5]=stairs_A_Offset;
        ep[6]=stairs_X_Offset;
        ep[7]=stairs_Y_Offset/2.0+stairs_Y_shift_amplitude*delta;
        ep[8]=stairs_Z_Offset;
        ep[9]=stairs_R_Offset;
        ep[10]=stairs_P_Offset;
        ep[11]=stairs_A_Offset;
        ram_data[DARWIN_STAIRS_CNTRL]|=SHIFT_WEIGHT_LEFT;
      }
      else if(stairs_Time>stairs_shift_weight_left_time && stairs_Time<=stairs_rise_right_foot_time)
      {
        //2
        delta=(stairs_Time-stairs_shift_weight_left_time)/(stairs_rise_right_foot_time-stairs_shift_weight_left_time);
        ep[0]=stairs_X_Offset;
        ep[1]=-stairs_Y_Offset/2.0+stairs_Y_shift_amplitude;
        ep[2]=stairs_Z_Offset;
        ep[3]=stairs_R_Offset-stairs_R_shift_amplitude*delta;
        ep[4]=stairs_P_Offset;
        ep[5]=stairs_A_Offset;
        ep[6]=stairs_X_Offset;
        ep[7]=stairs_Y_Offset/2.0+stairs_Y_shift_amplitude;
        ep[8]=stairs_Z_Offset+(stairs_Z_overshoot+stairs_Z_stair_height)*delta;
        ep[9]=stairs_R_Offset-stairs_R_shift_amplitude*delta;
        ep[10]=stairs_P_Offset;
        ep[11]=stairs_A_Offset;
        ram_data[DARWIN_STAIRS_CNTRL]|=RISE_RIGHT_FOOT;
      }
      else if(stairs_Time>stairs_rise_right_foot_time && stairs_Time<=stairs_advance_right_foot_time)
      {   
        //3
        delta=(stairs_Time-stairs_rise_right_foot_time)/(stairs_advance_right_foot_time-stairs_rise_right_foot_time);
        ep[0]=stairs_X_Offset;
        ep[1]=-stairs_Y_Offset/2.0+stairs_Y_shift_amplitude-(stairs_Y_spread_amplitude/2.0)*delta;
        ep[2]=stairs_Z_Offset;
        ep[3]=stairs_R_Offset-stairs_R_shift_amplitude;
        ep[4]=stairs_P_Offset+stairs_P_shift_amplitude*delta;
        ep[5]=stairs_A_Offset+stairs_A_shift_amplitude*delta;
        ep[6]=stairs_X_Offset+stairs_X_shift_amplitude*delta;
        ep[7]=stairs_Y_Offset/2.0+stairs_Y_shift_amplitude+(stairs_Y_spread_amplitude/2.0)*delta;
        ep[8]=stairs_Z_Offset+stairs_Z_overshoot+stairs_Z_stair_height;
        ep[9]=stairs_R_Offset-stairs_R_shift_amplitude;
        ep[10]=stairs_P_Offset+stairs_P_shift_amplitude*delta;
        ep[11]=stairs_A_Offset+stairs_A_shift_amplitude*delta;
        ram_data[DARWIN_STAIRS_CNTRL]|=ADVANCE_RIGHT_FOOT;
      }
      else if(stairs_Time>stairs_advance_right_foot_time && stairs_Time<=stairs_contact_right_foot_time)
      {
        //4
        delta=(stairs_Time-stairs_advance_right_foot_time)/(stairs_contact_right_foot_time-stairs_advance_right_foot_time);
        ep[0]=stairs_X_Offset;
        ep[1]=-stairs_Y_Offset/2.0+stairs_Y_shift_amplitude-(stairs_Y_spread_amplitude/2.0);
        ep[2]=stairs_Z_Offset;
        ep[3]=stairs_R_Offset-stairs_R_shift_amplitude;
        ep[4]=stairs_P_Offset+stairs_P_shift_amplitude;
        ep[5]=stairs_A_Offset+stairs_A_shift_amplitude;
        ep[6]=stairs_X_Offset+stairs_X_shift_amplitude;
        ep[7]=stairs_Y_Offset/2.0+stairs_Y_shift_amplitude+(stairs_Y_spread_amplitude/2.0);
        ep[8]=stairs_Z_Offset+stairs_Z_overshoot+stairs_Z_stair_height-stairs_Z_overshoot*delta;
        ep[9]=stairs_R_Offset-stairs_R_shift_amplitude;
        ep[10]=stairs_P_Offset+stairs_P_shift_amplitude;
        ep[11]=stairs_A_Offset+stairs_A_shift_amplitude;
        ram_data[DARWIN_STAIRS_CNTRL]|=CONTACT_RIGHT_FOOT;
      }
      else if(stairs_Time>stairs_contact_right_foot_time && stairs_Time<=stairs_shift_weight_right_time)
      {
        //5
        delta=(stairs_Time-stairs_contact_right_foot_time)/(stairs_shift_weight_right_time-stairs_contact_right_foot_time);
        ep[0]=stairs_X_Offset-stairs_X_shift_body*delta;
        ep[1]=-stairs_Y_Offset/2.0+stairs_Y_shift_amplitude-(stairs_Y_spread_amplitude/2.0)-(2*stairs_Y_shift_amplitude)*delta;
        ep[2]=stairs_Z_Offset;
        ep[3]=stairs_R_Offset-stairs_R_shift_amplitude+2.0*stairs_R_shift_amplitude*delta;
        ep[4]=stairs_P_Offset+stairs_P_shift_amplitude-3.0*stairs_P_shift_amplitude*delta;
        ep[5]=stairs_A_Offset+stairs_A_shift_amplitude;
        ep[6]=stairs_X_Offset+stairs_X_shift_amplitude-stairs_X_shift_body*delta;
        ep[7]=stairs_Y_Offset/2.0+stairs_Y_shift_amplitude+(stairs_Y_spread_amplitude/2.0)-(2*stairs_Y_shift_amplitude)*delta;
        ep[8]=stairs_Z_Offset+stairs_Z_stair_height;
        ep[9]=stairs_R_Offset-stairs_R_shift_amplitude+2.0*stairs_R_shift_amplitude*delta;
        ep[10]=stairs_P_Offset+stairs_P_shift_amplitude-3.0*stairs_P_shift_amplitude*delta;
        ep[11]=stairs_A_Offset+stairs_A_shift_amplitude;
        ram_data[DARWIN_STAIRS_CNTRL]|=SHIFT_WEIGHT_RIGHT;
      }
      else if(stairs_Time>stairs_shift_weight_right_time && stairs_Time<=stairs_rise_left_foot_time)
      {
        //6
        delta=(stairs_Time-stairs_shift_weight_right_time)/(stairs_rise_left_foot_time-stairs_shift_weight_right_time);
        ep[0]=stairs_X_Offset-stairs_X_shift_body;
        ep[1]=-stairs_Y_Offset/2.0-stairs_Y_shift_amplitude-(stairs_Y_spread_amplitude/2.0);
        ep[2]=stairs_Z_Offset+(stairs_Z_overshoot+stairs_Z_stair_height)*delta;
        ep[3]=stairs_R_Offset+stairs_R_shift_amplitude;
        ep[4]=stairs_P_Offset-2.0*stairs_P_shift_amplitude;
        ep[5]=stairs_A_Offset+stairs_A_shift_amplitude;
        ep[6]=stairs_X_Offset+stairs_X_shift_amplitude-stairs_X_shift_body;
        ep[7]=stairs_Y_Offset/2.0-stairs_Y_shift_amplitude+(stairs_Y_spread_amplitude/2.0);
        ep[8]=stairs_Z_Offset+stairs_Z_stair_height;
        ep[9]=stairs_R_Offset+stairs_R_shift_amplitude;
        ep[10]=stairs_P_Offset-2.0*stairs_P_shift_amplitude;
        ep[11]=stairs_A_Offset+stairs_A_shift_amplitude;
        ram_data[DARWIN_STAIRS_CNTRL]|=RISE_LEFT_FOOT;
      }
      else if(stairs_Time>stairs_rise_left_foot_time && stairs_Time<=stairs_advance_left_foot_time)
      {
        //7
        delta=(stairs_Time-stairs_rise_left_foot_time)/(stairs_advance_left_foot_time-stairs_rise_left_foot_time);
        ep[0]=stairs_X_Offset-stairs_X_shift_body+stairs_X_shift_body*delta;
        ep[1]=-stairs_Y_Offset/2.0-stairs_Y_shift_amplitude-(stairs_Y_spread_amplitude/2.0)+(stairs_Y_spread_amplitude/2.0)*delta;
        ep[2]=stairs_Z_Offset+stairs_Z_overshoot+stairs_Z_stair_height;
        ep[3]=stairs_R_Offset+stairs_R_shift_amplitude;
        ep[4]=stairs_P_Offset-2.0*stairs_P_shift_amplitude+stairs_P_shift_amplitude*delta;
        ep[5]=stairs_A_Offset+stairs_A_shift_amplitude-stairs_A_shift_amplitude*delta;
        ep[6]=stairs_X_Offset+stairs_X_shift_amplitude-stairs_X_shift_body-(stairs_X_shift_amplitude-stairs_X_shift_body)*delta;
        ep[7]=stairs_Y_Offset/2.0-stairs_Y_shift_amplitude+(stairs_Y_spread_amplitude/2.0)-(stairs_Y_spread_amplitude/2.0)*delta;
        ep[8]=stairs_Z_Offset+stairs_Z_stair_height;
        ep[9]=stairs_R_Offset+stairs_R_shift_amplitude;
        ep[10]=stairs_P_Offset-2.0*stairs_P_shift_amplitude+stairs_P_shift_amplitude*delta;
        ep[11]=stairs_A_Offset+stairs_A_shift_amplitude-stairs_A_shift_amplitude*delta;
        ram_data[DARWIN_STAIRS_CNTRL]|=ADVANCE_LEFT_FOOT;
      }
      else if(stairs_Time>stairs_advance_left_foot_time && stairs_Time<=stairs_contact_left_foot_time)
      {
        //8
        delta=(stairs_Time-stairs_advance_left_foot_time)/(stairs_contact_left_foot_time-stairs_advance_left_foot_time);
        ep[0]=stairs_X_Offset;
        ep[1]=-stairs_Y_Offset/2.0-stairs_Y_shift_amplitude;
        ep[2]=stairs_Z_Offset+stairs_Z_overshoot+stairs_Z_stair_height-stairs_Z_overshoot*delta;
        ep[3]=stairs_R_Offset+stairs_R_shift_amplitude-stairs_R_shift_amplitude*delta;
        ep[4]=stairs_P_Offset-stairs_P_shift_amplitude+stairs_P_shift_amplitude*delta;
        ep[5]=stairs_A_Offset;
        ep[6]=stairs_X_Offset;
        ep[7]=stairs_Y_Offset/2.0-stairs_Y_shift_amplitude;
        ep[8]=stairs_Z_Offset+stairs_Z_stair_height;
        ep[9]=stairs_R_Offset+stairs_R_shift_amplitude-stairs_R_shift_amplitude*delta;
        ep[10]=stairs_P_Offset-stairs_P_shift_amplitude+stairs_P_shift_amplitude*delta;
        ep[11]=stairs_A_Offset;
        ram_data[DARWIN_STAIRS_CNTRL]|=CONTACT_LEFT_FOOT;
      }
      else if(stairs_Time>stairs_contact_left_foot_time && stairs_Time<=stairs_center_weight_time)
      {
        //9
        delta=(stairs_Time-stairs_contact_left_foot_time)/(stairs_center_weight_time-stairs_contact_left_foot_time);
        ep[0]=stairs_X_Offset;
        ep[1]=-stairs_Y_Offset/2.0-stairs_Y_shift_amplitude+stairs_Y_shift_amplitude*delta;
        ep[2]=stairs_Z_Offset+stairs_Z_stair_height-stairs_Z_stair_height*delta;
        ep[3]=stairs_R_Offset;
        ep[4]=stairs_P_Offset;
        ep[5]=stairs_A_Offset;
        ep[6]=stairs_X_Offset;
        ep[7]=stairs_Y_Offset/2.0-stairs_Y_shift_amplitude+stairs_Y_shift_amplitude*delta;
        ep[8]=stairs_Z_Offset+stairs_Z_stair_height-stairs_Z_stair_height*delta;
        ep[9]=stairs_R_Offset;
        ep[10]=stairs_P_Offset;
        ep[11]=stairs_A_Offset;
        ram_data[DARWIN_STAIRS_CNTRL]|=CENTER_WEIGHT;
      }
      else
      {
        ram_data[DARWIN_STAIRS_CNTRL]&=(~STAIRS_STATUS);
        stairs_Ctrl_Running=0x00;
      }
    }
    else
    {   
      if(stairs_Time>=0 && stairs_Time<=stairs_shift_weight_left_time)
      {
        //1
        delta=stairs_Time/stairs_shift_weight_left_time;
        ep[0]=stairs_X_Offset;
        ep[1]=stairs_Y_Offset/2.0+stairs_Y_shift_amplitude*delta;
        ep[2]=stairs_Z_Offset+stairs_Z_stair_height*delta;
        ep[3]=stairs_R_Offset;
        ep[4]=stairs_P_Offset;
        ep[5]=stairs_A_Offset;
        ep[6]=stairs_X_Offset;
        ep[7]=stairs_Y_Offset/2.0+stairs_Y_shift_amplitude*delta;
        ep[8]=stairs_Z_Offset+stairs_Z_stair_height*delta;
        ep[9]=stairs_R_Offset;
        ep[10]=stairs_P_Offset;
        ep[11]=stairs_A_Offset;
        ram_data[DARWIN_STAIRS_CNTRL]|=SHIFT_WEIGHT_LEFT;
      }
      else if(stairs_Time>stairs_shift_weight_left_time && stairs_Time<=stairs_rise_right_foot_time)
      {
        //2
        delta=(stairs_Time-stairs_shift_weight_left_time)/(stairs_rise_right_foot_time-stairs_shift_weight_left_time);
        ep[0]=stairs_X_Offset;
        ep[1]=stairs_Y_Offset/2.0+stairs_Y_shift_amplitude;
        ep[2]=stairs_Z_Offset+stairs_Z_stair_height;
        ep[3]=stairs_R_Offset-stairs_R_shift_amplitude;
        ep[4]=stairs_P_Offset;
        ep[5]=stairs_A_Offset;
        ep[6]=stairs_X_Offset;
        ep[7]=stairs_Y_Offset/2.0+stairs_Y_shift_amplitude;
        ep[8]=stairs_Z_Offset+stairs_Z_stair_height+stairs_Z_overshoot*delta;
        ep[9]=stairs_R_Offset-stairs_R_shift_amplitude*delta;
        ep[10]=stairs_P_Offset;
        ep[11]=stairs_A_Offset;
        ram_data[DARWIN_STAIRS_CNTRL]|=RISE_RIGHT_FOOT;
      }
      else if(stairs_Time>stairs_rise_right_foot_time && stairs_Time<=stairs_advance_right_foot_time)
      {   
        //3
        delta=(stairs_Time-stairs_rise_right_foot_time)/(stairs_advance_right_foot_time-stairs_rise_right_foot_time);
        ep[0]=stairs_X_Offset;
        ep[1]=stairs_Y_Offset/2.0+stairs_Y_shift_amplitude-(stairs_Y_spread_amplitude/2.0)*delta;
        ep[2]=stairs_Z_Offset+stairs_Z_stair_height;
        ep[3]=stairs_R_Offset-stairs_R_shift_amplitude;
        ep[4]=stairs_P_Offset+stairs_P_shift_amplitude*delta;
        ep[5]=stairs_A_Offset+stairs_A_shift_amplitude*delta;
        ep[6]=stairs_X_Offset+stairs_X_shift_amplitude*delta;
        ep[7]=stairs_Y_Offset/2.0+stairs_Y_shift_amplitude+(stairs_Y_spread_amplitude/2.0)*delta;
        ep[8]=stairs_Z_Offset+stairs_Z_overshoot+stairs_Z_stair_height;
        ep[9]=stairs_R_Offset-stairs_R_shift_amplitude;
        ep[10]=stairs_P_Offset+stairs_P_shift_amplitude*delta;
        ep[11]=stairs_A_Offset+stairs_A_shift_amplitude*delta;
        ram_data[DARWIN_STAIRS_CNTRL]|=ADVANCE_RIGHT_FOOT;
      }
      else if(stairs_Time>stairs_advance_right_foot_time && stairs_Time<=stairs_contact_right_foot_time)
      {
        //4
        delta=(stairs_Time-stairs_advance_right_foot_time)/(stairs_contact_right_foot_time-stairs_advance_right_foot_time);
        ep[0]=stairs_X_Offset;
        ep[1]=stairs_Y_Offset/2.0+stairs_Y_shift_amplitude-(stairs_Y_spread_amplitude/2.0);
        ep[2]=stairs_Z_Offset+stairs_Z_stair_height+stairs_Z_overshoot*delta;
        ep[3]=stairs_R_Offset-stairs_R_shift_amplitude;
        ep[4]=stairs_P_Offset+stairs_P_shift_amplitude;
        ep[5]=stairs_A_Offset+stairs_A_shift_amplitude;
        ep[6]=stairs_X_Offset+stairs_X_shift_amplitude;
        ep[7]=stairs_Y_Offset/2.0+stairs_Y_shift_amplitude+(stairs_Y_spread_amplitude/2.0);
        ep[8]=stairs_Z_Offset+stairs_Z_overshoot+stairs_Z_stair_height-stairs_Z_stair_height*delta;
        ep[9]=stairs_R_Offset-stairs_R_shift_amplitude;
        ep[10]=stairs_P_Offset+stairs_P_shift_amplitude;
        ep[11]=stairs_A_Offset+stairs_A_shift_amplitude;
        ram_data[DARWIN_STAIRS_CNTRL]|=CONTACT_RIGHT_FOOT;
      }
      else if(stairs_Time>stairs_contact_right_foot_time && stairs_Time<=stairs_shift_weight_right_time)
      {
        //5
        delta=(stairs_Time-stairs_contact_right_foot_time)/(stairs_shift_weight_right_time-stairs_contact_right_foot_time);
        ep[0]=stairs_X_Offset-stairs_X_shift_body*delta;
        ep[1]=stairs_Y_Offset/2.0+stairs_Y_shift_amplitude-(stairs_Y_spread_amplitude/2.0)-(2*stairs_Y_shift_amplitude)*delta;
        ep[2]=stairs_Z_Offset+stairs_Z_stair_height+stairs_Z_overshoot;
        ep[3]=stairs_R_Offset-stairs_R_shift_amplitude+2.0*stairs_R_shift_amplitude*delta;
        ep[4]=stairs_P_Offset+stairs_P_shift_amplitude-(2.0*stairs_P_shift_amplitude)*delta;
        ep[5]=stairs_A_Offset+stairs_A_shift_amplitude;
        ep[6]=stairs_X_Offset+stairs_X_shift_amplitude-stairs_X_shift_body*delta;
        ep[7]=stairs_Y_Offset/2.0+stairs_Y_shift_amplitude+(stairs_Y_spread_amplitude/2.0)-(2*stairs_Y_shift_amplitude)*delta;
        ep[8]=stairs_Z_Offset+stairs_Z_overshoot;
        ep[9]=stairs_R_Offset-stairs_R_shift_amplitude+2.0*stairs_R_shift_amplitude*delta;
        ep[10]=stairs_P_Offset+stairs_P_shift_amplitude-(2.0*stairs_P_shift_amplitude)*delta;
        ep[11]=stairs_A_Offset+stairs_A_shift_amplitude;
        ram_data[DARWIN_STAIRS_CNTRL]|=SHIFT_WEIGHT_RIGHT;
      }
      else if(stairs_Time>stairs_shift_weight_right_time && stairs_Time<=stairs_rise_left_foot_time)
      {
        //6
        delta=(stairs_Time-stairs_shift_weight_right_time)/(stairs_rise_left_foot_time-stairs_shift_weight_right_time);
        ep[0]=stairs_X_Offset-stairs_X_shift_body;
        ep[1]=stairs_Y_Offset/2.0-stairs_Y_shift_amplitude-(stairs_Y_spread_amplitude/2.0);
        ep[2]=stairs_Z_Offset+stairs_Z_overshoot+stairs_Z_stair_height;
        ep[3]=stairs_R_Offset+stairs_R_shift_amplitude;
        ep[4]=stairs_P_Offset-1.0*stairs_P_shift_amplitude;
        ep[5]=stairs_A_Offset+stairs_A_shift_amplitude;
        ep[6]=stairs_X_Offset+stairs_X_shift_amplitude-stairs_X_shift_body;
        ep[7]=stairs_Y_Offset/2.0-stairs_Y_shift_amplitude+(stairs_Y_spread_amplitude/2.0);
        ep[8]=stairs_Z_Offset+stairs_Z_overshoot-stairs_Z_overshoot*delta;
        ep[9]=stairs_R_Offset+stairs_R_shift_amplitude;
        ep[10]=stairs_P_Offset-1.0*stairs_P_shift_amplitude;
        ep[11]=stairs_A_Offset+stairs_A_shift_amplitude;
        ram_data[DARWIN_STAIRS_CNTRL]|=RISE_LEFT_FOOT;
      }
      else if(stairs_Time>stairs_rise_left_foot_time && stairs_Time<=stairs_advance_left_foot_time)
      {
        //7
        delta=(stairs_Time-stairs_rise_left_foot_time)/(stairs_advance_left_foot_time-stairs_rise_left_foot_time);
        ep[0]=stairs_X_Offset-stairs_X_shift_body+stairs_X_shift_body*delta;
        ep[1]=stairs_Y_Offset/2.0-stairs_Y_shift_amplitude-(stairs_Y_spread_amplitude/2.0)+(stairs_Y_spread_amplitude/2.0)*delta;
        ep[2]=stairs_Z_Offset+stairs_Z_overshoot+stairs_Z_stair_height;
        ep[3]=stairs_R_Offset+stairs_R_shift_amplitude;//+stairs_R_shift_amplitude*delta;
        ep[4]=stairs_P_Offset-1.0*stairs_P_shift_amplitude+1.0*stairs_P_shift_amplitude*delta;
        ep[5]=stairs_A_Offset+stairs_A_shift_amplitude-stairs_A_shift_amplitude*delta;
        ep[6]=stairs_X_Offset+stairs_X_shift_amplitude-stairs_X_shift_body-(stairs_X_shift_amplitude-stairs_X_shift_body)*delta;
        ep[7]=stairs_Y_Offset/2.0-stairs_Y_shift_amplitude+(stairs_Y_spread_amplitude/2.0)-(stairs_Y_spread_amplitude/2.0)*delta;
        ep[8]=stairs_Z_Offset;
        ep[9]=stairs_R_Offset+stairs_R_shift_amplitude;//+stairs_R_shift_amplitude*delta;
        ep[10]=stairs_P_Offset-1.0*stairs_P_shift_amplitude+1.0*stairs_P_shift_amplitude*delta;
        ep[11]=stairs_A_Offset+stairs_A_shift_amplitude-stairs_A_shift_amplitude*delta;
        ram_data[DARWIN_STAIRS_CNTRL]|=ADVANCE_LEFT_FOOT;
      }
      else if(stairs_Time>stairs_advance_left_foot_time && stairs_Time<=stairs_contact_left_foot_time)
      {
        //8
        delta=(stairs_Time-stairs_advance_left_foot_time)/(stairs_contact_left_foot_time-stairs_advance_left_foot_time);
        ep[0]=stairs_X_Offset;
        ep[1]=stairs_Y_Offset/2.0-stairs_Y_shift_amplitude;
        ep[2]=stairs_Z_Offset+stairs_Z_overshoot+stairs_Z_stair_height-(stairs_Z_stair_height+stairs_Z_overshoot)*delta;
        ep[3]=stairs_R_Offset+stairs_R_shift_amplitude-stairs_R_shift_amplitude*delta;
        ep[4]=stairs_P_Offset;
        ep[5]=stairs_A_Offset;
        ep[6]=stairs_X_Offset;
        ep[7]=stairs_Y_Offset/2.0-stairs_Y_shift_amplitude;
        ep[8]=stairs_Z_Offset;
        ep[9]=stairs_R_Offset+stairs_R_shift_amplitude-stairs_R_shift_amplitude*delta;
        ep[10]=stairs_P_Offset;
        ep[11]=stairs_A_Offset;
        ram_data[DARWIN_STAIRS_CNTRL]|=CONTACT_LEFT_FOOT;
      }
      else if(stairs_Time>stairs_contact_left_foot_time && stairs_Time<=stairs_center_weight_time)
      {
        //9
        delta=(stairs_Time-stairs_contact_left_foot_time)/(stairs_center_weight_time-stairs_contact_left_foot_time);
        ep[0]=stairs_X_Offset;
        ep[1]=stairs_Y_Offset/2.0-stairs_Y_shift_amplitude+stairs_Y_shift_amplitude*delta;
        ep[2]=stairs_Z_Offset;
        ep[3]=stairs_R_Offset+stairs_R_shift_amplitude-stairs_R_shift_amplitude*delta;
        ep[4]=stairs_P_Offset;
        ep[5]=stairs_A_Offset;
        ep[6]=stairs_X_Offset;
        ep[7]=stairs_Y_Offset/2.0-stairs_Y_shift_amplitude+stairs_Y_shift_amplitude*delta;
        ep[8]=stairs_Z_Offset;
        ep[9]=stairs_R_Offset+stairs_R_shift_amplitude-stairs_R_shift_amplitude*delta;
        ep[10]=stairs_P_Offset;
        ep[11]=stairs_A_Offset;
        ram_data[DARWIN_STAIRS_CNTRL]|=CENTER_WEIGHT;
      }
      else
      {
        ram_data[DARWIN_STAIRS_CNTRL]&=(~STAIRS_STATUS);
        stairs_Ctrl_Running=0x00;
      }
    }
    
    if(ram_data[DARWIN_STAIRS_CNTRL]&STAIRS_STATUS)
      stairs_Time += stairs_period;
    else
      stairs_Time=0;

    // Compute angles with the inverse kinematics
    if((darwin_leg_ik(&angle[0],ep[0],ep[1],ep[2],ep[3],ep[4],ep[5])!=1) || 
       (darwin_leg_ik(&angle[6],ep[6],ep[7],ep[8],ep[9],ep[10],ep[11])!=1))
      return;// Do not use angles

    // Compute motor value
    if(manager_get_module(R_HIP_YAW)==MM_WALKING)
    {
      manager_current_angles[R_HIP_YAW]=((-180.0*angle[0])/PI)*65536.0;
      manager_current_slopes[R_HIP_YAW]=5;
    }
    if(manager_get_module(R_HIP_ROLL)==MM_WALKING)
    {
      manager_current_angles[R_HIP_ROLL]=((-180.0*angle[1])/PI)*65536.0;
      manager_current_slopes[R_HIP_ROLL]=5;
    }
    if(manager_get_module(R_HIP_PITCH)==MM_WALKING)
    {
      manager_current_angles[R_HIP_PITCH]=((180.0*angle[2])/PI-stairs_Hip_Pitch_Offset)*65536.0;
      manager_current_slopes[R_HIP_PITCH]=5;
    }
    if(manager_get_module(R_KNEE)==MM_WALKING)
    {
      manager_current_angles[R_KNEE]=((180.0*angle[3])/PI)*65536.0;
      manager_current_slopes[R_KNEE]=5;
    }
    if(manager_get_module(R_ANKLE_PITCH)==MM_WALKING)
    {
      manager_current_angles[R_ANKLE_PITCH]=((-180.0*angle[4])/PI)*65536.0;
      manager_current_slopes[R_ANKLE_PITCH]=5;
    }
    if(manager_get_module(R_ANKLE_ROLL)==MM_WALKING)
    {
      manager_current_angles[R_ANKLE_ROLL]=((180.0*angle[5])/PI)*65536.0;
      manager_current_slopes[R_ANKLE_ROLL]=5;
    }
    if(manager_get_module(L_HIP_YAW)==MM_WALKING)
    {
      manager_current_angles[L_HIP_YAW]=((-180.0*angle[6])/PI)*65536.0;
      manager_current_slopes[L_HIP_YAW]=5;
    }
    if(manager_get_module(L_HIP_ROLL)==MM_WALKING)
    {
      manager_current_angles[L_HIP_ROLL]=((-180.0*angle[7])/PI)*65536.0;
      manager_current_slopes[L_HIP_ROLL]=5;
    }
    if(manager_get_module(L_HIP_PITCH)==MM_WALKING)
    {
      manager_current_angles[L_HIP_PITCH]=((-180.0*angle[8])/PI+stairs_Hip_Pitch_Offset)*65536.0;
      manager_current_slopes[L_HIP_PITCH]=5;
    }
    if(manager_get_module(L_KNEE)==MM_WALKING)
    {
      manager_current_angles[L_KNEE]=((-180.0*angle[9])/PI)*65536.0;
      manager_current_slopes[L_KNEE]=5;
    }
    if(manager_get_module(L_ANKLE_PITCH)==MM_WALKING)
    {
      manager_current_angles[L_ANKLE_PITCH]=((180.0*angle[10])/PI)*65536.0;
      manager_current_slopes[L_ANKLE_PITCH]=5;
    }
    if(manager_get_module(L_ANKLE_ROLL)==MM_WALKING)
    {
      manager_current_angles[L_ANKLE_ROLL]=((180.0*angle[11])/PI)*65536.0;
      manager_current_slopes[L_ANKLE_ROLL]=5;
    }
  }
}

// operation functions
uint8_t stairs_in_range(unsigned short int address, unsigned short int length)
{
  if(ram_in_window(STAIRS_BASE_ADDRESS,STAIRS_MEM_LENGTH,address,length) ||
     ram_in_window(STAIRS_EEPROM_ADDRESS,STAIRS_EEPROM_LENGTH,address,length))
    return 0x01;
  else
    return 0x00;
}

void stairs_process_read_cmd(unsigned short int address,unsigned short int length,unsigned char *data)
{

}

void stairs_process_write_cmd(unsigned short int address,unsigned short int length,unsigned char *data)
{
  uint16_t i;

  // walk control
  if(ram_in_range(DARWIN_STAIRS_CNTRL,address,length))
  {
    if(data[DARWIN_STAIRS_CNTRL-address]&STAIRS_START_UP)
      stairs_start(0x01);
    if(data[DARWIN_STAIRS_CNTRL-address]&STAIRS_START_DOWN)
      stairs_start(0x00);
    if(data[DARWIN_STAIRS_CNTRL-address]&STAIRS_STOP)
      stairs_stop();
  }
  for(i=STAIRS_EEPROM_ADDRESS;i<=STAIRS_EEPROM_ADDRESS+STAIRS_EEPROM_LENGTH;i++)
    if(ram_in_range(i,address,length))
      ram_data[i]=data[i-address];
}

